@@ -4,7 +4,8 @@ exports.op = op;
 var Hash = require('./hash.js');
 var CodeRef = require('./code-ref.js');
 
-var LexPadHack = require('./lexpad-hack.js');
+var Ctx = require('./ctx.js');
+
 var NQPInt = require('./nqp-int.js');
 
 var NQPException = require('./nqp-exception.js');
@@ -13,7 +14,7 @@ var reprs = require('./reprs.js');
 
 var hll = require('./hll.js');
 
-var NQPArray = require('./array.js');
+var NQPObject = require('./nqp-object.js');
 
 var bootstrap = require('./bootstrap.js');
 
@@ -23,16 +24,16 @@ var constants = require('./constants.js');
 
 var containerSpecs = require('./container-specs.js');
 
+var Null = require('./null.js');
+
+var BOOT = require('./BOOT.js');
+
+var exceptionsStack = require('./exceptions-stack.js');
+
+var repossession = require('./repossession.js');
+
 exports.CodeRef = CodeRef;
 
-op.atpos = function(array, index) {
-  return array.$$atpos(index);
-};
-
-op.bindpos = function(array, index, value) {
-  return array.$$bindpos(index, value);
-};
-
 op.isinvokable = function(obj) {
   return (obj instanceof CodeRef || (obj._STable && obj._STable.invocationSpec) ? 1 : 0);
 };
@@ -49,13 +50,6 @@ op.escape = function(str) {
       .replace(/"/g, '\\"');
 };
 
-op.x = function(str, times) {
-  if (!times) return '';
-  var ret = str;
-  while (--times) ret += str;
-  return ret;
-};
-
 function radixHelper(radix, str, zpos, flags) {
   var lowercase = 'a-' + String.fromCharCode('a'.charCodeAt(0) + radix - 11);
   var uppercase = 'A-' + String.fromCharCode('A'.charCodeAt(0) + radix - 11);
@@ -81,13 +75,13 @@ function radixHelper(radix, str, zpos, flags) {
 
 exports.radixHelper = radixHelper;
 
-op.radix = function(radix, str, zpos, flags) {
+op.radix = function(hllName, radix, str, zpos, flags) {
   var extracted = radixHelper(radix, str, zpos, flags);
   if (extracted == null) {
-    return new NQPArray([0, 1, -1]);
+    return hll.slurpyArray(hllName, [0, 1, -1]);
   }
   var pow = Math.pow(radix, extracted.power);
-  return new NQPArray([parseInt(extracted.number, radix), pow, extracted.offset]);
+  return hll.slurpyArray(hllName, [parseInt(extracted.number, radix), pow, extracted.offset]);
 };
 
 op.setdebugtypename = function(type, debugName) {
@@ -95,72 +89,6 @@ op.setdebugtypename = function(type, debugName) {
   return type;
 };
 
-function Iter(array) {
-  this.array = array;
-  this.target = array.length;
-  this.idx = 0;
-}
-
-Iter.prototype.$$shift = function() {
-  return this.array[this.idx++];
-};
-
-Iter.prototype.$$toBool = function(ctx) {
-  return this.idx < this.target;
-};
-
-function HashIter(hash) {
-  this.hash = hash.content;
-  this.keys = Object.keys(hash.$$toObject());
-  this.target = this.keys.length;
-  this.idx = 0;
-}
-
-HashIter.prototype.$$shift = function() {
-  return new IterPair(this.hash, this.keys[this.idx++]);
-};
-
-HashIter.prototype.$$toBool = function(ctx) {
-  return this.idx < this.target;
-};
-
-function IterPair(hash, key) {
-  this._key = key;
-  this._hash = hash;
-}
-
-IterPair.prototype.iterval = function() {
-  return this._hash.get(this._key);
-};
-IterPair.prototype.iterkey_s = function() {
-  return this._key;
-};
-
-IterPair.prototype.Str = function(ctx) {
-  return this._key;
-};
-
-IterPair.prototype.key = function(ctx, named) {
-  return this._key;
-};
-IterPair.prototype.value = function(ctx, named) {
-  return this._hash.get(this._key);
-};
-
-
-op.iterator = function(obj) {
-  if (obj instanceof NQPArray) {
-    return new Iter(obj.array);
-  } else if (obj instanceof Hash) {
-    return new HashIter(obj);
-  } else if (obj instanceof LexPadHack) {
-    return new Iter(Object.keys(obj.content));
-  } else {
-    throw 'unsupported thing to iterate over';
-  }
-};
-
-
 exports.hash = function() {
   return new Hash();
 };
@@ -205,17 +133,13 @@ op.create = function(obj) {
   return obj._STable.REPR.allocate(obj._STable);
 };
 
-// HACK - till the array refactor we hack this
-
-op.bootarray = function(obj) {
-  return {_STable: {REPR: {allocate: function(STable) {
-    return new NQPArray([]);
-  }}}};
+op.bootarray = function() {
+  return BOOT.Array;
 };
 
 op.defined = function(obj) {
   // TODO - handle more things that aren't defined
-  if (obj === undefined || obj === null || obj.typeObject_) {
+  if (obj === Null || obj.typeObject_) {
     return 0;
   }
   return 1;
@@ -233,25 +157,21 @@ op.setboolspec = function(obj, mode, method) {
   return obj;
 };
 
-function Capture(named, pos, skip) {
-  this.pos = pos;
-  this.named = named;
-}
-
-Capture.prototype.$$clone = function() {
-  return this; // captures are immutable
+class Capture extends NQPObject {
+  constructor(named, pos, skip) {
+    super();
+    this.pos = pos;
+    this.named = named;
+  }
+  $$clone() {
+    return this;
+  }
 };
 
 op.savecapture = function(args) {
   return new Capture(args[1], Array.prototype.slice.call(args, 2));
 };
 
-op.savecaptureAsMethod = function(invocant, args) {
-  var withInvocant = Array.prototype.slice.call(args, 2);
-  withInvocant.unshift(invocant);
-  return new Capture(args[1], withInvocant);
-};
-
 op.captureposelems = function(capture) {
   return capture.pos.length;
 };
@@ -284,25 +204,10 @@ op.getcodeobj = function(codeRef) {
   return codeRef.codeObj;
 };
 
-// TODO benchmark and pick a fast way of doing this
-op.splice = function(target, source, offset, length) {
-  if (target.$$splice) return target.$$splice(source, offset, length);
-  var args = [offset, length];
-  for (var i = 0; i < source.length; i++) {
-    args.push(source[i]);
-  }
-  target.splice.apply(target, args);
-  return target;
-};
-
-op.findmethod = function(obj, method) {
-  return obj._STable.methodCache[method];
-};
 
 op.istype = function(ctx, obj, type) {
   /* Null always type checks false. */
-  /* HACK - undefined */
-  if (obj === null || obj === undefined) {
+  if (obj === Null) {
     return 0;
   }
 
@@ -311,16 +216,6 @@ op.istype = function(ctx, obj, type) {
     return 0;
   }
 
-  // HACK
-  if (obj instanceof NQPArray) {
-    if (hll.hllConfigs.nqp && type == hll.hllConfigs.nqp.get('list')) {
-      return 1;
-    } else {
-      return 0;
-    }
-  }
-
-  // TODO cases where the typeCheckCache isn't authoritative
   var cache = obj._STable.typeCheckCache;
   if (cache) {
     for (var i = 0; i < cache.length; i++) {
@@ -332,18 +227,34 @@ op.istype = function(ctx, obj, type) {
     /* If we get here, need to call .^type_check on the value we're
      * checking. */
 
+    var HOW = obj._STable.HOW;
     /* This "hack" is stolen from the JVM */
-    if (!obj._STable.HOW.type_check) {
+    if (!HOW.type_check) {
       return 0;
     }
 
-    return nqp.toBool(obj._STable.HOW.type_check(ctx, null, obj, type));
+    if (HOW.type_check(ctx, null, HOW, obj, type).$$toBool(ctx)) {
+      return 1;
+    }
   }
+
+  if (type._STable.typeCheckNeedsAccepts) {
+    return type._STable.HOW.accepts_type(ctx, null, type._STable.HOW, type, obj).$$toBool(ctx);
+  }
+
   return 0;
 };
 
 op.settypecache = function(obj, cache) {
   obj._STable.typeCheckCache = cache.array;
+  if (obj._STable._SC !== null) obj._STable.scwb();
+  return obj;
+};
+
+op.settypecheckmode = function(obj, mode) {
+  if (mode & 2) {
+    obj._STable.typeCheckNeedsAccepts = true;
+  }
   return obj;
 };
 
@@ -352,12 +263,17 @@ op.setmethcache = function(obj, cache) {
     console.log('we expect a hash here');
   }
   obj._STable.setMethodCache(cache.$$toObject());
+  if (obj._STable._SC !== null) obj._STable.scwb();
   return obj;
 };
 
 op.setmethcacheauth = function(obj, isAuth) {
-  /* TODO we currently assume method caches are always authorative
-    sadly that's not always the case*/
+  if (isAuth) {
+    obj._STable.modeFlags |= constants.METHOD_CACHE_AUTHORITATIVE;
+  } else {
+    obj._STable.modeFlags &= ~constants.METHOD_CACHE_AUTHORITATIVE;
+  }
+  if (obj._STable._SC !== null) obj._STable.scwb();
   return obj;
 };
 
@@ -387,8 +303,37 @@ op.newtype = function(how, repr) {
   return REPR.typeObjectFor(how);
 };
 
-op.can = function(obj, method) {
-  return obj._STable.methodCache.hasOwnProperty(method) ? 1 : 0;
+function find_method(ctx, obj, name) {
+  if (obj._STable.methodCache) {
+    var hasMethod = obj._STable.methodCache.hasOwnProperty(name);
+    if (hasMethod) {
+      return obj._STable.methodCache[name];
+    }
+    if (obj._STable.modeFlags & constants.METHOD_CACHE_AUTHORITATIVE) {
+      return Null;
+    }
+  }
+
+  if (obj._STable.HOW.find_method) {
+    return obj._STable.HOW.find_method(ctx, null, obj._STable.HOW, obj, name);
+  } else {
+    return Null;
+  }
+}
+
+op.can = function(ctx, obj, name) {
+  /* HACK those things should have an STable rather then be special cased */
+  if (typeof obj !== 'object' || obj instanceof NQPInt || obj instanceof CodeRef || obj instanceof Hash) return 0;
+
+  return find_method(ctx, obj, name) === Null ? 0 : 1;
+};
+
+op.findmethod = function(ctx, obj, name) {
+  var method = find_method(ctx, obj, name);
+  if (method === Null) {
+    throw new NQPException("Cannot find method '" + name + "' on object of type " + obj._STable.debugName);
+  }
+  return method;
 };
 
 op.getcodename = function(code) {
@@ -397,10 +342,12 @@ op.getcodename = function(code) {
 
 op.setcodename = function(code, name) {
   code.name = name;
+  return code;
 };
 
 op.rebless = function(obj, newType) {
   obj._STable.REPR.changeType(obj, newType);
+  if (obj._SC !== null) obj.$$scwb();
   return obj;
 };
 
@@ -420,7 +367,7 @@ op.clone = function(obj) {
 
 var whereCounter = 0;
 op.where = function(obj) {
-  if (obj._STable) { // HACK
+  if (obj._STable || obj instanceof CodeRef) { // HACK
     if (!obj._WHERE) {
       obj._WHERE = ++whereCounter;
     }
@@ -455,21 +402,23 @@ op.setcontspec = function(type, specType, conf) {
 };
 
 op.iscont = function(cont) {
-  return cont.$$decont ? 1 : 0;
+  return cont.$$iscont ? cont.$$iscont() : 0;
 };
 
 op.iscont_i = function(cont) {
-  // Stub
+  if (cont.$$iscont_i) return cont.$$iscont_i();
   return 0;
 };
 
 op.iscont_n = function(cont) {
-  // Stub
+  if (cont.$$iscont_n) return cont.$$iscont_n();
   return 0;
 };
 
 op.iscont_s = function(cont) {
-  // Stub
+  if (cont.$$iscont_s) {
+    return cont.$$iscont_s();
+  }
   return 0;
 };
 
@@ -477,11 +426,6 @@ op.isrwcont = function(cont) {
   return cont.$$decont ? 1 : 0;
 };
 
-op.decont = function(ctx, cont) {
-  if (cont === null || cont === undefined) return cont;
-  return cont.$$decont ? cont.$$decont(ctx) : cont;
-};
-
 op.box_n = function(n, type) {
   var repr = type._STable.REPR;
   var obj = repr.allocate(type._STable);
@@ -530,15 +474,21 @@ op.freshcoderef = function(code) {
   return fresh;
 };
 
-/* TODO - make serialization take this into account */
-var compilingSCs = [];
 op.pushcompsc = function(sc) {
-  compilingSCs.push(sc);
+  repossession.compilingSCs.push(sc);
   return sc;
 };
 
 op.popcompsc = function(sc) {
-  return compilingSCs.pop();
+  return repossession.compilingSCs.pop();
+};
+
+op.scwbenable = function() {
+  return --repossession.scwbDisableDepth;
+};
+
+op.scwbdisable = function() {
+  return ++repossession.scwbDisableDepth;
 };
 
 var compilerRegistry = new Map();
@@ -548,20 +498,23 @@ op.bindcomp = function(language, compiler) {
   return compiler;
 };
 
-function WrappedFunction(func) {
-  this.func = func;
-}
-
-WrappedFunction.prototype.$apply = function(args) {
-  var converted = [];
-  for (var i = 2; i < args.length; i++) {
-    converted.push(toJS(args[i]));
+class WrappedFunction extends NQPObject {
+  constructor(func) {
+    super();
+    this.func = func;
   }
-  return fromJS(this.func.apply(null, converted));
-};
 
-WrappedFunction.prototype.$call = function(args) {
-  return this.$apply(arguments);
+  $$apply(args) {
+    var converted = [];
+    for (var i = 2; i < args.length; i++) {
+      converted.push(toJS(args[i]));
+    }
+    return fromJS(this.func.apply(null, converted));
+  }
+
+  $$call(args) {
+    return this.$$apply(arguments);
+  }
 };
 
 function fromJS(obj) {
@@ -581,33 +534,45 @@ function toJS(obj) {
       for (var i = 0; i < arguments.length; i++) {
         converted.push(fromJS(arguments[i]));
       }
-      return toJS(obj.$apply(converted));
+      return toJS(obj.$$apply(converted));
     };
   } else {
     return obj;
   }
 }
 
-compilerRegistry.set('JavaScript', {
-  eval: function(ctx, named, code) {
+class JavaScriptCompiler extends NQPObject {
+  eval(ctx, _NAMED, self, code) {
+    if (/PRINTME/.test(code)) {
+      console.log('evaling [', code, ']');
+    }
     //console.log("evaling [", code, "]");
     return fromJS(eval(code));
   }
-});
+};
 
-compilerRegistry.set('nqp', {
-  backend: function(ctx, named) {
-    return {
-      name: function(ctx, named) {
-        return 'js';
-      }
-    };
+compilerRegistry.set('JavaScript', new JavaScriptCompiler());
+
+class JSBackendStub extends NQPObject {
+  name(ctx, named) {
+    return 'js';
   }
-});
+};
 
+class NQPStub extends NQPObject {
+  constructor() {
+    super();
+    this.backend_ = new JSBackendStub();
+  }
+  backend(ctx, named) {
+    return this.backend_;
+  }
+};
+
+compilerRegistry.set('nqp', new NQPStub());
 
 op.getcomp = function(language) {
-  return compilerRegistry.has(language) ? compilerRegistry.get(language) : null;
+  return compilerRegistry.has(language) ? compilerRegistry.get(language) : Null;
 };
 
 op.backendconfig = function() {
@@ -633,7 +598,7 @@ op.setmessage = function(exception, message) {
 };
 
 op.getpayload = function(exception) {
-  return exception.payload;
+  return Object.prototype.hasOwnProperty.call(exception, 'payload') ? exception.payload : Null;
 };
 
 op.setpayload = function(exception, payload) {
@@ -655,8 +620,6 @@ function renameEncoding(encoding) {
 exports.renameEncoding = renameEncoding;
 
 function byteSize(buf) {
-  if (buf.bytes) return buf.bytes;
-
   var bits = buf._STable.REPR.type._STable.REPR.bits;
 
   if (bits % 8) {
@@ -666,9 +629,6 @@ function byteSize(buf) {
   return bits / 8;
 }
 
-// HACK should be using buf instead of creating a new one
-// TODO needs to be fixed after an Array handling refactor
-
 op.encode = function(str, encoding_, buf) {
   var encoding = renameEncoding(encoding_);
 
@@ -680,13 +640,11 @@ op.encode = function(str, encoding_, buf) {
 
   var offset = 0;
   for (var i = 0; i < buffer.length / elementSize; i++) {
-    ret[i] = buffer.readIntLE(offset, elementSize);
+    buf.array[i] = buffer.readIntLE(offset, elementSize);
     offset += elementSize;
   }
 
-  ret = new NQPArray(ret);
-  ret.bytes = elementSize;
-  return ret;
+  return buf;
 };
 
 op.decode = function(buf, encoding_) {
@@ -707,7 +665,7 @@ op.decode = function(buf, encoding_) {
 };
 
 op.objprimspec = function(obj) {
-  if (obj === null) return 0;
+  if (obj === Null) return 0;
   return (obj._STable && obj._STable.REPR.boxedPrimitive ? obj._STable.REPR.boxedPrimitive : 0);
 };
 
@@ -717,10 +675,10 @@ op.setparameterizer = function(ctx, type, parameterizer) {
   /* Ensure that the type is not already parametric or parameterized. */
   if (st.parameterizer) {
     ctx.die('This type is already parametric');
-    return null;
+    return Null;
   } else if (st.parametricType) {
     ctx.die('Cannot make a parameterized type also be parametric');
-    return null;
+    return Null;
   }
 
   /* Set up the type as parameterized. */
@@ -759,7 +717,7 @@ op.parameterizetype = function(ctx, type, params) {
     }
   }
 
-  var result = st.parameterizer.$call(ctx, {}, st.WHAT, params);
+  var result = st.parameterizer.$$call(ctx, {}, st.WHAT, params);
 
   var newSTable = result._STable;
   newSTable.parametricType = type;
@@ -824,7 +782,7 @@ op.typeparameterat = function(ctx, type, idx) {
 op.typeparameterized = function(type) {
   var st = type._STable;
   var nqp = require('nqp-runtime');
-  return st.parametricType ? st.parametricType : null;
+  return st.parametricType ? st.parametricType : Null;
 };
 
 function startTrampoline(thunk_) {
@@ -838,10 +796,10 @@ var resetValue;
 var invokeValue;
 op.continuationreset = function(ctx, tag, continuation) {
   startTrampoline(function() {
-    return continuation.$callCPS(ctx, {}, function(value) {
+    return continuation.$$callCPS(ctx, {}, function(value) {
       resetValue = value;
       invokeValue = value;
-      return null;
+      return Null;
     });
   });
   return resetValue;
@@ -852,23 +810,23 @@ op.continuationresetCPS = function(ctx, tag, continuation, continuation) {
 };
 
 op.continuationcontrolCPS = function(ctx, protect, tag, run, cont) {
-  startTrampoline(run.$callCPS(ctx, {}, function(value) {
+  startTrampoline(run.$$callCPS(ctx, {}, function(value) {
     resetValue = value;
-    return null;
+    return Null;
   }, cont));
-  return null;
+  return Null;
 };
 
 op.continuationinvoke = function(ctx, cont, inject) {
   // TODO really place inject inside the cont
-  var value = inject.$call(ctx, {});
+  var value = inject.$$call(ctx, {});
   startTrampoline(cont(value));
   return invokeValue;
 };
 
 op.continuationinvokeCPS = function(ctx, invokedCont, inject, cont) {
   // TODO really place inject inside the cont, use callCPS
-  var value = inject.$call(ctx, {});
+  var value = inject.$$call(ctx, {});
   return invokedCont(value);
 };
 
@@ -888,7 +846,6 @@ op.getlexrel = function(pad, name) {
   return pad.lookup(name);
 };
 
-//TODO think about polyfill for codePointAt, fromCodePoint
 
 op.bitand_s = function(a, b) {
   var ret = '';
@@ -976,19 +933,19 @@ op.setdimensions = function(array, dimensions) {
   };
 
   op['atpos2d' + type] = function(array, x, y) {
-    return op['atposnd' + type](array, new NQPArray([x, y]));
+    return op['atposnd' + type](array, BOOT.createArray([x, y]));
   };
 
   op['atpos3d' + type] = function(array, x, y, z) {
-    return op['atposnd' + type](array, new NQPArray([x, y, z]));
+    return op['atposnd' + type](array, BOOT.createArray([x, y, z]));
   };
 
   op['bindpos2d' + type] = function(array, x, y, value) {
-    return op['bindposnd' + type](array, new NQPArray([x, y]), value);
+    return op['bindposnd' + type](array, BOOT.createArray([x, y]), value);
   };
 
   op['bindpos3d' + type] = function(array, x, y, z, value) {
-    return op['bindposnd' + type](array, new NQPArray([x, y, z]), value);
+    return op['bindposnd' + type](array, BOOT.createArray([x, y, z]), value);
   };
 });
 
@@ -1022,87 +979,8 @@ op.getstaticcode = function(codeRef) {
   return codeRef.staticCode;
 };
 
-
-var MAX_ARITY = 4;
-var MAX_PER_ARITY = 16;
-
-function MultiCache() {
-  this.cache = [];
-  for (var i = 0; i < MAX_ARITY; i++) {
-    this.cache[i] = [];
-  }
-}
-
-function posTypes(capture) {
-  var arity = capture.pos.length;
-  var types = new Array(arity);
-  for (var i = 0; i < arity; i++) {
-    var obj = capture.pos[i];
-    if (obj._STable) {
-      types[i] = obj._STable;
-    } else if (obj instanceof NQPInt) {
-      types[i] = 1;
-    } else if (typeof obj == 'number') {
-      types[i] = 2;
-    } else if (typeof obj == 'string') {
-      types[i] = 3;
-    }
-  }
-  return types;
-}
-
-op.multicachefind = function(cache, capture) {
-  if (!cache) return null;
-  var arity = capture.pos.length;
-  var hasNamed = capture.named ? true : false;
-
-  if (arity == 0) {
-    if (!hasNamed && cache.zeroArity) {
-      return cache.zeroArity;
-    } else {
-      return null;
-    }
-  }
-
-  if (arity > MAX_ARITY) return null;
-
-  var types = posTypes(capture);
-
-  var arityCache = cache.cache[arity - 1];
-
-  CANDIDATES: for (var i = 0; i < arityCache.length; i++) {
-    for (var j = 0; j < arityCache[i].types.length; j++) {
-      if (arityCache[i].types[j] !== types[j]) continue CANDIDATES;
-    }
-    if (arityCache[i].hasNamed !== hasNamed) continue CANDIDATES;
-    return arityCache[i].result;
-  }
-
-  return null;
-};
-
-op.multicacheadd = function(cache, capture, result) {
-  var c = cache ? cache : new MultiCache();
-  var arity = capture.pos.length;
-  var hasNamed = capture.named ? true : false;
-
-  if (arity == 0) {
-    if (!hasNamed) {
-      c.zeroArity = result;
-    }
-    return c;
-  }
-
-  if (arity > MAX_ARITY || c.cache[arity - 1].length > MAX_PER_ARITY) {
-    return c;
-  }
-
-  c.cache[arity - 1].push({types: posTypes(capture), hasNamed: hasNamed, result: result});
-  return c;
-};
-
-op.backtracestrings = function(exception) {
-  return new NQPArray(['backtrace NYI']);
+op.backtracestrings = function(hllName, exception) {
+  return hll.list(hllName, [exception.stack.replace(/^Error: .*\n/, '')]);
 };
 
 op.hintfor = function(classHandle, attrName) {
@@ -1111,9 +989,111 @@ op.hintfor = function(classHandle, attrName) {
 };
 
 op.ctxcaller = function(ctx) {
-  return ctx.caller;
+  return ctx.$$caller;
 };
 
 op.captureposprimspec = function(capture, idx) {
   return 0;
 };
+
+op.forceouterctx = function(code, ctx) {
+  if (!(code instanceof CodeRef)) {
+    throw 'forceouterctx first operand must be a CodeRef';
+  }
+  if (!(ctx instanceof Ctx)) {
+    throw 'forceouterctx second operand must be a Ctx';
+  }
+
+  code.forcedOuterCtx = ctx;
+
+  return code;
+};
+
+op.getattrref_i = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('int_attr_ref');
+  if (refType === undefined) {
+    throw 'No int attribute reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+op.getattrref_n = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('num_attr_ref');
+  if (refType === undefined) {
+    throw 'No num attribute reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+op.getattrref_s = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('str_attr_ref');
+  if (refType === undefined) {
+    throw 'No str attribute reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+op.getlexref_i = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('int_lex_ref');
+  if (refType === undefined) {
+    throw 'No int lexical reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+op.getlexref_s = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('str_lex_ref');
+  if (refType === undefined) {
+    throw 'No str lexical reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+op.getlexref_n = function(hllName, get, set) {
+  var refType = hll.hllConfigs[hllName].get('num_lex_ref');
+  if (refType === undefined) {
+    throw 'No num lexical reference type registered for current HLL';
+  }
+  var STable = refType._STable;
+  var ref = STable.REPR.allocate(STable);
+  ref.get = get;
+  ref.set = set;
+  return ref;
+};
+
+// TODO: replace it with a fully correct implementation
+op.fc = function(string) {
+  return string.toLowerCase();
+};
+
+op.islist = function(list) {
+  return (list._STable && list._STable.REPR instanceof reprs.VMArray) ? 1 : 0;
+};
+
+op.split = function(hllName, separator, string) {
+  return hll.slurpyArray(hllName, string !== '' ? string.split(separator) : []);
+};
+
+op.exception = function() {
+  return exceptionsStack[exceptionsStack.length - 1];
+};
