@@ -1,31 +1,20 @@
+'use strict';
 var op = {};
 exports.op = op;
 
 var iscclass = require('./cclass.js').op.iscclass;
 var nqp = require('nqp-runtime');
 
-var NQPArray = require('./array.js');
+var Null = require('./null.js');
+var BOOT = require('./BOOT.js');
 
-var EDGE_FATE = 0,
-    EDGE_EPSILON = 1,
-    EDGE_CODEPOINT = 2,
-    EDGE_CODEPOINT_NEG = 3,
-    EDGE_CHARCLASS = 4,
-    EDGE_CHARCLASS_NEG = 5,
-    EDGE_CHARLIST = 6,
-    EDGE_CHARLIST_NEG = 7,
-    EDGE_SUBRULE = 8,
-    EDGE_CODEPOINT_I = 9,
-    EDGE_CODEPOINT_I_NEG = 10,
-    EDGE_GENERIC_VAR = 11,
-    EDGE_CHARRANGE = 12,
-    EDGE_CHARRANGE_NEG = 13,
-    EDGE_CODEPOINT_LL = 14,
-    EDGE_CODEPOINT_I_LL = 15;
+const EDGE_FATE = 0, EDGE_EPSILON = 1, EDGE_CODEPOINT = 2, EDGE_CODEPOINT_NEG = 3, EDGE_CHARCLASS = 4, EDGE_CHARCLASS_NEG = 5;
+const EDGE_CHARLIST = 6, EDGE_CHARLIST_NEG = 7, EDGE_SUBRULE = 8, EDGE_CODEPOINT_I = 9, EDGE_CODEPOINT_I_NEG = 10;
+const EDGE_GENERIC_VAR = 11, EDGE_CHARRANGE = 12, EDGE_CHARRANGE_NEG = 13, EDGE_CODEPOINT_LL = 14, EDGE_CODEPOINT_I_LL = 15;
 
 function convertState(thing) {
-  if (thing instanceof NQPArray) {
-    return thing.array.map(function(x) { return convertState(x) });
+  if (thing.$$toArray) {
+    return thing.$$toArray().map(function(x) { return convertState(x) });
   } else {
     return thing;
   }
@@ -262,17 +251,21 @@ function runNFA(nfa, target, pos) {
     }
   }
 
-  return new NQPArray(fates);
+  return BOOT.createArray(fates);
 }
 
 op.nfarunproto = function(nfa, target, pos) {
   return runNFA(nfa, target, pos);
 };
 
-op.nfarunalt = function(nfa, target, pos, bstack, cstack, marks) {
-  cstack = cstack instanceof NQPArray ? cstack.array : cstack;
-  bstack = bstack instanceof NQPArray ? bstack.array : bstack;
-  marks = marks instanceof NQPArray ? marks.array : marks;
+op.nfarunalt = function(nfa, target, pos, bstackWrapped, cstackWrapped, marksWrapped) {
+  var cstack;
+
+  if (cstackWrapped !== Null && !cstackWrapped.typeObject_) {
+    cstack = cstackWrapped.$$toArray();
+  }
+  var bstack = bstackWrapped.$$toArray();
+  var marks = marksWrapped.$$toArray();
 
   /* Run the NFA. */
   var fates = runNFA(nfa, target, pos).array;
