@@ -1,8 +1,13 @@
+'use strict';
 var op = {};
 exports.op = op;
 
 var NQPInt = require('./nqp-int.js');
 var NQPException = require('./nqp-exception.js');
+
+var null_s = require('./null_s.js');
+var Null = require('./null.js');
+
 exports.NQPInt = NQPInt;
 
 function loadOps(module) {
@@ -37,6 +42,8 @@ loadOps(cclass);
 var hll = require('./hll.js');
 loadOps(hll);
 
+loadOps(require('./multicache.js'));
+
 var deserialization = require('./deserialization.js');
 exports.wval = deserialization.wval;
 loadOps(deserialization);
@@ -47,69 +54,66 @@ loadOps(serialization);
 var nativecall = require('./nativecall.js');
 loadOps(nativecall);
 
-exports.CodeRef = require('./code-ref.js');
+var CodeRef = require('./code-ref.js');
+exports.CodeRef = CodeRef;
 
 exports.CurLexpad = require('./curlexpad.js');
 
 var Hash = require('./hash.js');
 
+var Ctx = require('./ctx.js');
+module.exports.Ctx = Ctx;
+
+module.exports.CtxWithStatic = require('./ctx-with-static.js');
+
 var bootstrap = require('./bootstrap.js');
 module.exports.knowhowattr = bootstrap.knowhowattr;
 module.exports.knowhow = bootstrap.knowhow;
 
-module.exports.NQPArray = require('./array.js');
-
-var saveCtxAs;
-var savedCtxs = {};
-
-function saveCtx(where, block) {
-  var old = saveCtxAs;
-  saveCtxAs = where;
-  block();
-  saveCtxAs = old;
-}
-
-exports.loadModule = function(module, helper) {
-  saveCtx(module, function() {
-    module = module.replace(/::/g, '/');
-    if (helper) {
-      helper();
-    } else {
-      require(module);
-    }
-  });
-};
-
-exports.loadSetting = exports.loadModule;
-
-exports.setupSetting = function(settingName) {
-  return savedCtxs[settingName + '.setting'];
-};
-
-exports.ctxsave = function(ctx) {
-  savedCtxs[saveCtxAs] = ctx;
-};
-
-
-var LexPadHack = require('./lexpad-hack.js');
+exports.loaderCtx = null;
 
 op.loadbytecode = function(ctx, file) {
   // HACK - temporary hack for rakudo-js
   if (file == '/share/nqp/lib/Perl6/BOOTSTRAP.js') {
     file = 'Perl6::BOOTSTRAP';
   }
-  exports.loadModule(file);
-  // HACK - ctx is sometimes NULL on rakudo-js
-  if (ctx) ctx.bindDynamic('$*MAIN_CTX', new LexPadHack(savedCtxs[file]));
+
+  var loadFrom;
+  if (ctx && ((loadFrom = ctx.lookupDynamic('$*LOADBYTECODE_FROM')) !== Null)) {
+  } else {
+    loadFrom = module;
+  }
+
+  var oldLoaderCtx = exports.loaderCtx;
+  exports.loaderCtx = ctx;
+  var mangled = file.replace(/::/g, '-');
+
+  var prefixes = (process.env.NQPJS_LIB || '').split(':');
+  prefixes.push('./', 'nqp-js-on-js/');
+  var found = false;
+  for (var prefix of prefixes) {
+    try {
+      loadFrom.require(prefix + mangled);
+      found = true;
+      break;
+    } catch (e) {
+      if (e.code !== 'MODULE_NOT_FOUND') {
+        throw e;
+      }
+    }
+  }
+  if (!found) throw "can't find: " + file + ', looking in: ' + prefixes.join(', ') + ' from ' + loadFrom.filename;
+  exports.loaderCtx = oldLoaderCtx;
+
   return file;
 };
 
 op.ctxlexpad = function(ctx) {
-  // HACK
-  if (ctx instanceof LexPadHack) {
+  if (ctx instanceof Ctx) {
     return ctx;
   } else {
-    console.log('ctxlexpad NYI');
+    console.trace('ctxlexpad needs a ctx as an argument');
+    throw 'ctxlexpad needs a ctx as an argument';
   }
 };
 
@@ -118,23 +122,26 @@ op.lexprimspec = function(pad, key) {
   return 0;
 };
 
-op.ctxouter = function(hack) {
-  return null;
+op.ctxouter = function(ctx) {
+  return ctx.$$outer === null ? Null : ctx.$$outer;
 };
 
-exports.toStr = function(arg, ctx) {
+exports.toStr = function(arg_, ctx) {
+  var arg = arg_.$$decont(ctx);
   if (typeof arg == 'number') {
     return arg.toString();
   } else if (typeof arg == 'string') {
     return arg;
-  } else if (arg === null) {
+  } else if (arg === Null) {
+    return '';
+  } else if (arg === null_s) {
     return arg;
-  } else if (arg !== undefined && arg !== null && arg.typeObject_) {
+  } else if (arg.typeObject_) {
     return '';
-  } else if (arg.Str) {
-    return arg.Str(ctx);
   } else if (arg.$$getStr) {
     return arg.$$getStr();
+  } else if (arg.Str) {
+    return arg.Str(ctx, null, arg);
   } else if (arg.$$getNum) {
     return arg.$$getNum().toString();
   } else if (arg.$$getInt) {
@@ -147,18 +154,31 @@ exports.toStr = function(arg, ctx) {
 exports.toNum = function(arg, ctx) {
   if (typeof arg == 'number') {
     return arg;
+  } else if (arg === Null) {
+    return 0;
   } else if (typeof arg == 'string') {
     var ret = parseFloat(arg);
     return isNaN(ret) ? 0 : ret;
+  } else if (arg._STable && arg._STable.methodCache && arg._STable.methodCache.Num) {
+    var result = arg.Num(ctx, null, arg);
+    if (result.$$getNum) {
+      return result.$$getNum();
+    } else if (result.$$numify) {
+      return result.$$numify();
+    } else if (typeof result == 'number') {
+      return result;
+    } else {
+      console.trace("we can't numify result of toNum");
+      process.exit();
+    }
   } else if (arg.typeObject_) {
-    // TODO - is that a correct way to do that?
     return 0;
-  } else if (arg.Num) {
-    return arg.Num(ctx);
   } else if (arg.$$getNum) {
     return arg.$$getNum();
   } else if (arg.$$getInt) {
     return arg.$$getInt();
+  } else if (arg.$$numify) {
+    return arg.$$numify();
   } else {
     console.log(arg);
     throw "Can't convert to num";
@@ -166,172 +186,54 @@ exports.toNum = function(arg, ctx) {
 };
 
 exports.toInt = function(arg, ctx) {
-  if (typeof arg == 'number') {
-    return arg | 0;
-  } else if (arg.Int) {
-    return arg.Int(ctx);
-  } else if (typeof arg == 'string') {
-    var ret = parseInt(arg);
-    return isNaN(ret) ? 0 : ret;
-  } else if (arg.typeObject_) {
-    return 0;
-  } else {
-    throw "Can't convert to int";
+  return (exports.toNum(arg, ctx) | 0);
+};
+
+exports.intToObj = function(hllName, i) {
+  var currentHLL = hll.hllConfigs[hllName];
+  var type;
+  if (currentHLL) type = currentHLL.get('int_box');
+  if (!type) {
+    return new NQPInt(i);
+  }
+  else {
+    var repr = type._STable.REPR;
+    var obj = repr.allocate(type._STable);
+    obj.$$setInt(i);
+    return obj;
   }
 };
 
-exports.toBool = function(arg, ctx) {
-  if (typeof arg == 'number') {
-    return arg ? 1 : 0;
-  } else if (typeof arg == 'string') {
-    return arg == '' ? 0 : 1;
-  } else if (arg === undefined || arg == null) {
-    return 0;
-  } else if (arg.$$toBool) {
-    return arg.$$toBool(ctx);
-  } else if (typeof arg == 'function') {
-    // needed for continuations
-    return 1;
-  } else {
-    throw "Can't decide if arg is true";
+exports.numToObj = function(hllName, n) {
+  var currentHLL = hll.hllConfigs[hllName];
+  var type;
+  if (currentHLL) type = currentHLL.get('num_box');
+  if (!type) {
+    return n;
+  }
+  else {
+    var repr = type._STable.REPR;
+    var obj = repr.allocate(type._STable);
+    obj.$$setNum(n);
+    return obj;
   }
 };
 
-function Ctx(callerCtx, outerCtx, codeRef) {
-  this.caller = callerCtx;
-  this.outer = outerCtx;
-  this.codeRef = codeRef;
-}
-
-
-
-Ctx.prototype.propagateException = function(exception) {
-  var ctx = this;
-  while (ctx) {
-    if (ctx.CATCH) {
-      exception.caught = ctx;
-      exception.resume = false;
-      ctx.exception = exception;
-      ctx.unwind.ret = ctx.CATCH();
-      if (exception.resume) {
-        return;
-      } else {
-        throw ctx.unwind;
-      }
-    }
-    ctx = ctx.caller;
+exports.strToObj = function(hllName, s) {
+  var currentHLL = hll.hllConfigs[hllName];
+  var type;
+  if (currentHLL) type = currentHLL.get('str_box');
+  if (!type) {
+    return s;
   }
-  throw exception.message;
-};
-
-Ctx.prototype.catchException = function(exception) {
-  this.exception = exception;
-  this.CATCH();
-};
-
-Ctx.prototype.rethrow = function(exception) {
-  this.propagateException(exception);
-};
-
-Ctx.prototype.die = function(msg) {
-  this.propagateException(new NQPException(msg));
-};
-
-Ctx.prototype.resume = function(exception) {
-  exception.resume = true;
-};
-
-Ctx.prototype.throw = function(exception) {
-  this.propagateException(exception);
-};
-
-Ctx.prototype.lookupDynamic = function(name) {
-  var ctx = this;
-  while (ctx) {
-    if (ctx.hasOwnProperty(name)) {
-      return ctx[name];
-    }
-    ctx = ctx.caller;
+  else {
+    var repr = type._STable.REPR;
+    var obj = repr.allocate(type._STable);
+    obj.$$setStr(s);
+    return obj;
   }
-  return null;
-  /* Looking up of a contextual is allowed to fail,
-     nqp code usually fallbacks to looking up of global */
 };
 
-Ctx.prototype.lookupDynamicFromCaller = function(name) {
-  var ctx = this.caller;
-  while (ctx) {
-    if (ctx.hasOwnProperty(name)) {
-      return ctx[name];
-    }
-    ctx = ctx.caller;
-  }
-  return null;
-  /* Looking up of a contextual is allowed to fail,
-     nqp code usually fallbacks to looking up of global */
-};
-
-Ctx.prototype.lookupFromSomeCaller = function(name) {
-  var currentCallerCtx = this.caller;
-  while (currentCallerCtx) {
-    var currentCtx = currentCallerCtx;
-    while (currentCtx) {
-      if (currentCtx.hasOwnProperty(name)) {
-        return currentCtx[name];
-      }
-      currentCtx = currentCtx.outer;
-    }
-    currentCallerCtx = currentCallerCtx.caller;
-  }
-  return null;
-};
-
-Ctx.prototype.lookup = function(name) {
-  var ctx = this;
-  while (ctx) {
-    if (ctx.hasOwnProperty(name)) {
-      return ctx[name];
-    }
-    ctx = ctx.outer;
-  }
-  /* Rakudo depends on returning null when we can't lookup a lexical */
-  return null;
-};
-
-Ctx.prototype.$$atkey = function(key) {
-  return this.lookup(key);
-};
-
-Ctx.prototype.$$bindkey = function(key, value) {
-  this[key] = value;
-};
-
-Ctx.prototype.bind = function(name, value) {
-  var ctx = this;
-  while (ctx) {
-    if (ctx.hasOwnProperty(name)) {
-      ctx[name] = value;
-      return ctx[name];
-    }
-    ctx = ctx.outer;
-  }
-  throw "Can't bind: " + name;
-};
-
-Ctx.prototype.bindDynamic = function(name, value) {
-  var ctx = this;
-  while (ctx) {
-    if (ctx.hasOwnProperty(name)) {
-      ctx[name] = value;
-      return ctx[name];
-    }
-    ctx = ctx.caller;
-  }
-  throw "Can't bind dynamic: " + name;
-};
-
-exports.Ctx = Ctx;
-
 if (!Math.imul) {
   /* Polyfill from:
   https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/imul
@@ -424,15 +326,6 @@ exports.NYI = function(msg) {
   return null;
 };
 
-/* HACK - needed until we handle types on attributes */
-exports.intAttrHack = function(attrValue) {
-  if (attrValue instanceof exports.NQPInt) {
-    return attrValue.value;
-  } else {
-    return attrValue;
-  }
-};
-
 exports.args = function(module) {
   return require.main === module ? process.argv.slice(1) : [];
 };
@@ -448,6 +341,54 @@ exports.runCPS = runCPS;
 exports.NQPException = NQPException;
 
 exports.wrapException = function(e) {
-  console.log(e);
   return new NQPException(e.message);
 };
+
+function ControlReturn(payload) {
+  this.payload = payload;
+}
+exports.ControlReturn = ControlReturn;
+
+function PassExceptionToCaller(exception) {
+  this.exception = exception;
+}
+exports.PassExceptionToCaller = PassExceptionToCaller;
+
+exports.setCodeRefHLL = function(codeRefs, hllName) {
+  for (var i = 0; i < codeRefs.length; i++) {
+    codeRefs[i].hll = hll.hllConfigs[hllName];
+  }
+};
+
+
+/* TODO - make monkey patching builtin things optional */
+
+Number.prototype.$$decont = function(ctx) {
+  return this;
+};
+
+Number.prototype.$$toBool = function(ctx) {
+  return this === 0 ? 0 : 1;
+};
+
+String.prototype.$$decont = function(ctx) {
+  return this;
+};
+
+String.prototype.$$toBool = function(ctx) {
+  return this === '' ? 0 : 1;
+};
+
+// needed for continuations
+Function.prototype.$$toBool = function(ctx) {
+  return 1;
+};
+
+
+exports.null_s = null_s;
+exports.Null = Null;
+
+exports.list = hll.list;
+
+exports.slurpyArray = hll.slurpyArray;
+exports.createArray = require('./BOOT').createArray;
