@@ -3,7 +3,9 @@ package org.perl6.nqp.io;
 import java.io.File;
 import java.io.IOException;
 import java.nio.channels.FileChannel;
+import java.nio.channels.FileLock;
 import java.nio.charset.Charset;
+import java.nio.file.Files;
 import java.nio.file.OpenOption;
 import java.nio.file.Path;
 import java.nio.file.StandardOpenOption;
@@ -13,9 +15,10 @@ import java.util.List;
 import org.perl6.nqp.runtime.ExceptionHandling;
 import org.perl6.nqp.runtime.ThreadContext;
 
-public class FileHandle extends SyncHandle implements IIOSeekable {
+public class FileHandle extends SyncHandle implements IIOSeekable, IIOLockable {
     
     FileChannel fc;
+    FileLock lock;
     protected boolean eof = false;
     
     public static OpenOption[] resolveOpenMode(String mode) {
@@ -55,6 +58,8 @@ public class FileHandle extends SyncHandle implements IIOSeekable {
     public FileHandle(ThreadContext tc, String filename, String mode) {
         try {
             Path p = new File(filename).toPath();
+            if (Files.isDirectory(p))
+                ExceptionHandling.dieInternal(tc, "Tried to open directory " + filename);
             OpenOption[] opts = resolveOpenMode(mode);
             if(opts == null)
                 ExceptionHandling.dieInternal(tc, "Unhandled file open mode '" + mode + "'");
@@ -86,6 +91,8 @@ public class FileHandle extends SyncHandle implements IIOSeekable {
         } catch (IllegalArgumentException e) {
             throw ExceptionHandling.dieInternal(tc, e);
         }
+        /* Reset readBuffer since content is out of sync after fc.position. */
+        readBuffer = null;
     }
     
     public long tell(ThreadContext tc) {
@@ -96,6 +103,40 @@ public class FileHandle extends SyncHandle implements IIOSeekable {
             throw ExceptionHandling.dieInternal(tc, e);
         }
     }
+
+    public void lock(ThreadContext tc, long flag) {
+        if (lock != null && lock.acquiredBy() == fc) {
+            /* XXX: *might* not be quite the exact condition we want,
+             *      which more precisely would be "are we the thread that locked?"
+             *      but the Lock doesn't know which Thread it came from, afaict.
+             *      In any case, to match nqp-m behavior, we just don't do anything.
+             */
+        }
+        else {
+            try {
+                lock = fc.lock();
+            } catch (IOException e) {
+                throw ExceptionHandling.dieInternal(tc, e);
+            } catch (IllegalArgumentException e) {
+                throw ExceptionHandling.dieInternal(tc, e);
+            } catch (IllegalStateException e) {
+                throw ExceptionHandling.dieInternal(tc, e);
+            }
+        }
+    }
+
+    public void unlock(ThreadContext tc) {
+        try {
+            if (lock != null) {
+                lock.release();
+                lock = null;
+            }
+        } catch (IOException e) {
+            throw ExceptionHandling.dieInternal(tc, e);
+        } catch (IllegalArgumentException e) {
+            throw ExceptionHandling.dieInternal(tc, e);
+        }
+    }
     
     public void flush(ThreadContext tc) {
         try {
