@@ -15,14 +15,15 @@ public class ExceptionHandling {
     public static final int EX_CAT_NEXT = 4;
     public static final int EX_CAT_REDO = 8;
     public static final int EX_CAT_LAST = 16;
-    public static final int EX_CAT_TAKE = 32;
-    public static final int EX_CAT_WARN = 64;
-    public static final int EX_CAT_SUCCEED = 128;
+    public static final int EX_CAT_RETURN = 32;
+    public static final int EX_CAT_TAKE = 128;
+    public static final int EX_CAT_WARN = 256;
+    public static final int EX_CAT_SUCCEED = 512;
+    public static final int EX_CAT_PROCEED = 1024;
     public static final int EX_CAT_LABELED = 4096;
     public static final int EX_CAT_AWAIT = 8192;
     public static final int EX_CAT_EMIT = 16384;
     public static final int EX_CAT_DONE = 32768;
-    public static final int EX_CAT_RETURN = 65536;
     
     /* Exception handler kinds. */
     public static final int EX_UNWIND_SIMPLE = 0;
@@ -162,6 +163,10 @@ all:
         }
         if (handler != null)
             invokeHandler(tc, handler, category, f, false, exObj, null);
+        else if (tc.curFrame.codeRef.staticInfo.compUnit.hllConfig.lexicalHandlerNotFoundError != null) {
+            Ops.invokeDirect(tc, tc.curFrame.codeRef.staticInfo.compUnit.hllConfig.lexicalHandlerNotFoundError, 
+                Ops.intIntCallSite, false, new Object[] { category, 0L });
+        }
         else
             panic(tc, category, exObj);
     }
@@ -304,11 +309,11 @@ all:
             String kls = info.compUnit.getClass().getName();
             String method = info.methodName;
 
-            while (jcursor < ex.nativeTrace.length && !kls.equals(ex.nativeTrace[jcursor].getClassName()) &&
+            while (ex.nativeTrace != null && jcursor < ex.nativeTrace.length && !kls.equals(ex.nativeTrace[jcursor].getClassName()) &&
                         (method == null || !method.equals(ex.nativeTrace[jcursor].getMethodName())))
                 jcursor++;
 
-            StackTraceElement el = jcursor < ex.nativeTrace.length ? ex.nativeTrace[jcursor++] : null;
+            StackTraceElement el = ex.nativeTrace != null && jcursor < ex.nativeTrace.length ? ex.nativeTrace[jcursor++] : null;
 
             result.add(new TraceElement(ncursor, el != null ? el.getFileName() : null, el != null ? el.getLineNumber() : -1));
             ncursor = ncursor.caller;
