@@ -20,6 +20,17 @@ public class NFA extends REPR {
     public static final int EDGE_CHARLIST_NEG     = 7;
     public static final int EDGE_CODEPOINT_I      = 9;
     public static final int EDGE_CODEPOINT_I_NEG  = 10;
+    public static final int EDGE_GENERIC_VAR      = 11;
+    public static final int EDGE_CHARRANGE        = 12;
+    public static final int EDGE_CHARRANGE_NEG    = 13;
+    public static final int EDGE_CODEPOINT_LL     = 14;
+    public static final int EDGE_CODEPOINT_I_LL   = 15;
+    public static final int EDGE_CODEPOINT_M      = 16;
+    public static final int EDGE_CODEPOINT_M_NEG  = 17;
+    public static final int EDGE_CODEPOINT_M_LL   = 18;
+    public static final int EDGE_CODEPOINT_IM     = 19;
+    public static final int EDGE_CODEPOINT_IM_NEG = 20;
+    public static final int EDGE_CODEPOINT_IM_LL  = 21;
     
     public SixModelObject type_object_for(ThreadContext tc, SixModelObject HOW) {
         STable st = new STable(this, HOW);
@@ -66,8 +77,9 @@ public class NFA extends REPR {
                     body.states[i][j] = new NFAStateInfo();
                     body.states[i][j].act = (int)reader.readLong();
                     body.states[i][j].to = (int)reader.readLong();
-                    switch (body.states[i][j].act) {
+                    switch (body.states[i][j].act & 0xff) {
                     case EDGE_FATE:
+                    case EDGE_CODEPOINT_LL:
                     case EDGE_CODEPOINT:
                     case EDGE_CODEPOINT_NEG:
                     case EDGE_CHARCLASS:
@@ -78,8 +90,11 @@ public class NFA extends REPR {
                     case EDGE_CHARLIST_NEG:
                         body.states[i][j].arg_s = reader.readStr();
                         break;
+                    case EDGE_CODEPOINT_I_LL:
                     case EDGE_CODEPOINT_I:
-                    case EDGE_CODEPOINT_I_NEG: {
+                    case EDGE_CODEPOINT_I_NEG:
+                    case EDGE_CHARRANGE:
+                    case EDGE_CHARRANGE_NEG: {
                         body.states[i][j].arg_lc = (char)reader.readLong();
                         body.states[i][j].arg_uc = (char)reader.readLong();
                         break;
@@ -108,8 +123,9 @@ public class NFA extends REPR {
             for (int j = 0; j < body.states[i].length; j++) {
                 writer.writeInt(body.states[i][j].act);
                 writer.writeInt(body.states[i][j].to);
-                switch (body.states[i][j].act) {
+                switch (body.states[i][j].act & 0xff) {
                 case EDGE_FATE:
+                case EDGE_CODEPOINT_LL:
                 case EDGE_CODEPOINT:
                 case EDGE_CODEPOINT_NEG:
                 case EDGE_CHARCLASS:
@@ -120,8 +136,11 @@ public class NFA extends REPR {
                 case EDGE_CHARLIST_NEG:
                     writer.writeStr(body.states[i][j].arg_s);
                     break;
+                case EDGE_CODEPOINT_I_LL:
                 case EDGE_CODEPOINT_I:
-                case EDGE_CODEPOINT_I_NEG: {
+                case EDGE_CODEPOINT_I_NEG:
+                case EDGE_CHARRANGE:
+                case EDGE_CHARRANGE_NEG: {
                     writer.writeInt(body.states[i][j].arg_lc);
                     writer.writeInt(body.states[i][j].arg_uc);
                     break;
